@@ -35,12 +35,12 @@
     int iter;
     double temp;
     int msum, iflag;
-    double delta;
+    double delta = 0.;
     int jeval;
     int ncsuc;
     double ratio;
     double fnorm;
-    double pnorm, xnorm, fnorm1;
+    double pnorm, xnorm = 0., fnorm1;
     int nslow1, nslow2;
     int ncfail;
     double actred, epsmch, prered;
