@@ -26,7 +26,6 @@
 #define p0001 1e-4
 
     /* System generated locals */
-    int fjac_dim1, fjac_offset;
     double d1, d2;
 
     /* Local variables */
@@ -36,9 +35,9 @@
     int iter;
     double temp, temp1, temp2;
     int iflag;
-    double delta;
+    double delta = 0.;
     double ratio;
-    double fnorm, gnorm, pnorm, xnorm, fnorm1, actred, dirder, 
+    double fnorm, gnorm, pnorm, xnorm = 0., fnorm1, actred, dirder, 
 	    epsmch, prered;
     int info;
 
@@ -218,21 +217,6 @@
 /*     jorge j. more */
 
 /*     ********** */
-    /* Parameter adjustments */
-    --wa4;
-    --fvec;
-    --wa3;
-    --wa2;
-    --wa1;
-    --qtf;
-    --ipvt;
-    --diag;
-    --x;
-    fjac_dim1 = ldfjac;
-    fjac_offset = 1 + fjac_dim1 * 1;
-    fjac -= fjac_offset;
-
-    /* Function Body */
 
 /*     epsmch is the machine precision. */
 
@@ -250,7 +234,7 @@
 	goto TERMINATE;
     }
     if (mode == 2) {
-        for (j = 1; j <= n; ++j) {
+        for (j = 0; j < n; ++j) {
             if (diag[j] <= 0.) {
                 goto TERMINATE;
             }
@@ -260,12 +244,12 @@
 /*     evaluate the function at the starting point */
 /*     and calculate its norm. */
 
-    iflag = (*fcn)(p, m, n, &x[1], &fvec[1], &wa3[1], 1);
+    iflag = (*fcn)(p, m, n, x, fvec, wa3, 1);
     *nfev = 1;
     if (iflag < 0) {
 	goto TERMINATE;
     }
-    fnorm = enorm(m, &fvec[1]);
+    fnorm = enorm(m, fvec);
 
 /*     initialize levenberg-marquardt parameter and iteration counter. */
 
@@ -281,7 +265,7 @@
         if (nprint > 0) {
             iflag = 0;
             if ((iter - 1) % nprint == 0) {
-                iflag = (*fcn)(p, m, n, &x[1], &fvec[1], &wa3[1], 0);
+                iflag = (*fcn)(p, m, n, x, fvec, wa3, 0);
             }
             if (iflag < 0) {
                 goto TERMINATE;
@@ -293,20 +277,20 @@
 /*        forming (q transpose)*fvec and storing the first */
 /*        n components in qtf. */
 
-        for (j = 1; j <= n; ++j) {
+        for (j = 0; j < n; ++j) {
             qtf[j] = 0.;
-            for (i = 1; i <= n; ++i) {
-                fjac[i + j * fjac_dim1] = 0.;
+            for (i = 0; i < n; ++i) {
+                fjac[i + j * ldfjac] = 0.;
             }
         }
         iflag = 2;
-        for (i = 1; i <= m; ++i) {
-            if ((*fcn)(p, m, n, &x[1], &fvec[1], &wa3[1], iflag) < 0) {
+        for (i = 0; i < m; ++i) {
+            if ((*fcn)(p, m, n, x, fvec, wa3, iflag) < 0) {
                 goto TERMINATE;
             }
             temp = fvec[i];
-            rwupdt(n, &fjac[fjac_offset], ldfjac, &wa3[1], &qtf[1], &temp,
-                   &wa1[1], &wa2[1]);
+            rwupdt(n, fjac, ldfjac, wa3, qtf, &temp,
+                   wa1, wa2);
             ++iflag;
         }
         ++(*njev);
@@ -315,28 +299,28 @@
 /*        reorder its columns and update the components of qtf. */
 
         sing = FALSE_;
-        for (j = 1; j <= n; ++j) {
-            if (fjac[j + j * fjac_dim1] == 0.) {
+        for (j = 0; j < n; ++j) {
+            if (fjac[j + j * ldfjac] == 0.) {
                 sing = TRUE_;
             }
-            ipvt[j] = j;
-            wa2[j] = enorm(j, &fjac[j * fjac_dim1 + 1]);
+            ipvt[j] = j+1;
+            wa2[j] = enorm(j+1, &fjac[j * ldfjac + 0]);
         }
         if (sing) {
-            qrfac(n, n, &fjac[fjac_offset], ldfjac, TRUE_, &ipvt[1], n,
-                  &wa1[1], &wa2[1], &wa3[1]);
-            for (j = 1; j <= n; ++j) {
-                if (fjac[j + j * fjac_dim1] != 0.) {
+            qrfac(n, n, fjac, ldfjac, TRUE_, ipvt, n,
+                  wa1, wa2, wa3);
+            for (j = 0; j < n; ++j) {
+                if (fjac[j + j * ldfjac] != 0.) {
                     sum = 0.;
-                    for (i = j; i <= n; ++i) {
-                        sum += fjac[i + j * fjac_dim1] * qtf[i];
+                    for (i = j; i < n; ++i) {
+                        sum += fjac[i + j * ldfjac] * qtf[i];
                     }
-                    temp = -sum / fjac[j + j * fjac_dim1];
-                    for (i = j; i <= n; ++i) {
-                        qtf[i] += fjac[i + j * fjac_dim1] * temp;
+                    temp = -sum / fjac[j + j * ldfjac];
+                    for (i = j; i < n; ++i) {
+                        qtf[i] += fjac[i + j * ldfjac] * temp;
                     }
                 }
-                fjac[j + j * fjac_dim1] = wa1[j];
+                fjac[j + j * ldfjac] = wa1[j];
             }
         }
 
@@ -345,7 +329,7 @@
 
         if (iter == 1) {
             if (mode != 2) {
-                for (j = 1; j <= n; ++j) {
+                for (j = 0; j < n; ++j) {
                     diag[j] = wa2[j];
                     if (wa2[j] == 0.) {
                         diag[j] = 1.;
@@ -356,10 +340,10 @@
 /*        on the first iteration, calculate the norm of the scaled x */
 /*        and initialize the step bound delta. */
 
-            for (j = 1; j <= n; ++j) {
+            for (j = 0; j < n; ++j) {
                 wa3[j] = diag[j] * x[j];
             }
-            xnorm = enorm(n, &wa3[1]);
+            xnorm = enorm(n, wa3);
             delta = factor * xnorm;
             if (delta == 0.) {
                 delta = factor;
@@ -370,12 +354,12 @@
 
         gnorm = 0.;
         if (fnorm != 0.) {
-            for (j = 1; j <= n; ++j) {
-                l = ipvt[j];
+            for (j = 0; j < n; ++j) {
+                l = ipvt[j]-1;
                 if (wa2[l] != 0.) {
                     sum = 0.;
-                    for (i = 1; i <= j; ++i) {
-                        sum += fjac[i + j * fjac_dim1] * (qtf[i] / fnorm);
+                    for (i = 0; i <= j; ++i) {
+                        sum += fjac[i + j * ldfjac] * (qtf[i] / fnorm);
                     }
                     /* Computing MAX */
                     d1 = fabs(sum / wa2[l]);
@@ -396,7 +380,7 @@
 /*        rescale if necessary. */
 
         if (mode != 2) {
-            for (j = 1; j <= n; ++j) {
+            for (j = 0; j < n; ++j) {
                 /* Computing MAX */
                 d1 = diag[j], d2 = wa2[j];
                 diag[j] = max(d1,d2);
@@ -409,17 +393,17 @@
 
 /*           determine the levenberg-marquardt parameter. */
 
-            lmpar(n, &fjac[fjac_offset], ldfjac, &ipvt[1], &diag[1], &qtf[1], delta,
-                  &par, &wa1[1], &wa2[1], &wa3[1], &wa4[1]);
+            lmpar(n, fjac, ldfjac, ipvt, diag, qtf, delta,
+                  &par, wa1, wa2, wa3, wa4);
 
 /*           store the direction p and x + p. calculate the norm of p. */
 
-            for (j = 1; j <= n; ++j) {
+            for (j = 0; j < n; ++j) {
                 wa1[j] = -wa1[j];
                 wa2[j] = x[j] + wa1[j];
                 wa3[j] = diag[j] * wa1[j];
             }
-            pnorm = enorm(n, &wa3[1]);
+            pnorm = enorm(n, wa3);
 
 /*           on the first iteration, adjust the initial step bound. */
 
@@ -429,12 +413,12 @@
 
 /*           evaluate the function at x + p and calculate its norm. */
 
-            iflag = (*fcn)(p, m, n, &wa2[1], &wa4[1], &wa3[1], 1);
+            iflag = (*fcn)(p, m, n, wa2, wa4, wa3, 1);
             ++(*nfev);
             if (iflag < 0) {
                 goto TERMINATE;
             }
-            fnorm1 = enorm(m, &wa4[1]);
+            fnorm1 = enorm(m, wa4);
 
 /*           compute the scaled actual reduction. */
 
@@ -448,15 +432,15 @@
 /*           compute the scaled predicted reduction and */
 /*           the scaled directional derivative. */
 
-            for (j = 1; j <= n; ++j) {
+            for (j = 0; j < n; ++j) {
                 wa3[j] = 0.;
-                l = ipvt[j];
+                l = ipvt[j]-1;
                 temp = wa1[l];
-                for (i = 1; i <= j; ++i) {
-                    wa3[i] += fjac[i + j * fjac_dim1] * temp;
+                for (i = 0; i <= j; ++i) {
+                    wa3[i] += fjac[i + j * ldfjac] * temp;
                 }
             }
-            temp1 = enorm(n, &wa3[1]) / fnorm;
+            temp1 = enorm(n, wa3) / fnorm;
             temp2 = (sqrt(par) * pnorm) / fnorm;
             prered = temp1 * temp1 + temp2 * temp2 / p5;
             dirder = -(temp1 * temp1 + temp2 * temp2);
@@ -497,14 +481,14 @@
 
 /*           successful iteration. update x, fvec, and their norms. */
 
-                for (j = 1; j <= n; ++j) {
+                for (j = 0; j < n; ++j) {
                     x[j] = wa2[j];
                     wa2[j] = diag[j] * x[j];
                 }
-                for (i = 1; i <= m; ++i) {
+                for (i = 0; i < m; ++i) {
                     fvec[i] = wa4[i];
                 }
-                xnorm = enorm(n, &wa2[1]);
+                xnorm = enorm(n, wa2);
                 fnorm = fnorm1;
                 ++iter;
             }
@@ -557,7 +541,7 @@ TERMINATE:
 	info = iflag;
     }
     if (nprint > 0) {
-	(*fcn)(p, m, n, &x[1], &fvec[1], &wa3[1], 0);
+	(*fcn)(p, m, n, x, fvec, wa3, 0);
     }
     return info;
 
